/*------------------------------------------------------------------------------
 * MDK Middleware - Component ::USB:Host
 * Copyright (c) 2004-2015 ARM Germany GmbH. All rights reserved.
 *------------------------------------------------------------------------------
 * Name:    Keyboard.c
 * Purpose: USB Host HID Keyboard example program
 *----------------------------------------------------------------------------*/

#include <stdio.h>                      /* standard I/O .h-file               */
#include <ctype.h>                      /* character functions                */
#include "cmsis_os.h"                   /* CMSIS RTOS definitions             */

#include "samv71.h"                     // Device header
#include "rl_usb.h"                     // Keil.MDK-Pro::USB:CORE
#include "Terminal.h"

/*------------------------------------------------------------------------------
 *        USB Host Thread
 *----------------------------------------------------------------------------*/
void USBH_Thread (void const *arg) {
  osPriority priority;                       /* Thread priority               */
  char con    =  0;                          /* Connection status of kbd      */
  char con_ex =  40;                         /* Previous connection status
                                                + initial time in 100 ms
                                                intervals for initial disp    */
  char out    =  1;                          /* Output to keyboard LEDs       */

  USBH_Initialize (0);                       /* Initialize USB Host 0         */

  while (1) {
    con = USBH_HID_GetDeviceStatus(0) == usbOK;  /* Get kbd connection status */
    if ((con ^ con_ex) & 1) {                /* If connection status changed  */
      priority = osThreadGetPriority (osThreadGetId());
      osThreadSetPriority (osThreadGetId(), osPriorityAboveNormal);
      if (con) {
        USBH_HID_Write (0,(uint8_t *)&out,1);/* Turn on NUM LED               */
        printf ("\nKeyboard connected\n");
      } else {
        printf ("\nKeyboard disconnected ...\n");
      }
      osThreadSetPriority (osThreadGetId(), priority);
      con_ex = con;
    } else if (con_ex > 1) {                 /* If initial time active        */
      con_ex -= 2;                           /* Decrement initial time        */
      if ((con_ex <= 1) && (!con)) {         /* If initial time expired       */
        priority = osThreadGetPriority (osThreadGetId());
        osThreadSetPriority (osThreadGetId(), osPriorityAboveNormal);
        printf ("\nNo keyboard connected ... \n");
        osThreadSetPriority (osThreadGetId(), priority);
        con_ex = con;
      } else {
        osDelay(200);
      }
    }
    osDelay(100);
  }
}
osThreadDef(USBH_Thread, osPriorityNormal, 1, NULL);

/*------------------------------------------------------------------------------
 *        Application
 *----------------------------------------------------------------------------*/
int main (void) {
  char buf [15];                     

  /* Disable watchdog */
  WDT->WDT_MR = WDT_MR_WDDIS;
	
  /* Update system clock */
  SystemCoreClockUpdate();
  
  /* Enable data cache */
  SCB_EnableDCache();	
  SCB_EnableICache();
  
  osThreadCreate (osThread(USBH_Thread), NULL); 

  while (1)  {                               /* loop forever                  */
    getline (&buf[0], sizeof (buf));
  }
}
